package com.fjhx.purchase.service.pay.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.account.entity.account.po.AccountPayment;
import com.fjhx.account.service.account.AccountPaymentService;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.purchase.entity.pay.po.Pay;
import com.fjhx.purchase.entity.purchase.vo.PurchaseVo;
import com.fjhx.purchase.mapper.pay.PayMapper;
import com.fjhx.purchase.service.pay.PayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.supply.entity.supplier.po.SupplierInfo;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.purchase.entity.pay.vo.PayVo;
import com.fjhx.purchase.entity.pay.dto.PaySelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.purchase.entity.pay.dto.PayDto;
import cn.hutool.core.bean.BeanUtil;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购付款 服务实现类
 * </p>
 *
 * @author
 * @since 2023-04-13
 */
@Service
public class PayServiceImpl extends ServiceImpl<PayMapper, Pay> implements PayService {

    @Autowired
    private SupplierInfoService supplierInfoService;

    @Autowired
    private AccountPaymentService accountPaymentService;

    /**
     * 分页
     * @param dto
     * @return
     */
    @Override
    public Page<PayVo> getPage(PaySelectDto dto) {
        IWrapper<Pay> wrapper = getWrapper();
        wrapper.orderByDesc("p", Pay::getCreateTime);

        //供应商ID
        wrapper.eq("p",Pay::getSupplyId,dto.getSupplyId());

        //付款方式
        wrapper.eq("p",Pay::getPayType,dto.getPayType());

        //申请金额
        wrapper.ge("p",Pay::getAmount,dto.getStartAmount());
        wrapper.le("p",Pay::getAmount,dto.getEndAmount());

        //申请时间
        wrapper.ge("p", Pay::getCreateTime,dto.getBeginTime());
        wrapper.le("p",Pay::getCreateTime,dto.getEndTime());

        if(StringUtils.isNotEmpty(dto.getStatus())){//审批状态
            wrapper.eq("p",Pay::getStatus,dto.getStatus());
        }

        if(StringUtils.isNotEmpty(dto.getPayStatus())){//付款状态
            wrapper.eq("p",Pay::getPayStatus,dto.getPayStatus());
        }
        if(StringUtils.isNotEmpty(dto.getKeyword())){
            wrapper.keyword(dto.getKeyword(),new SqlField(Pay::getUserName),new SqlField(Pay::getRemark));
        }
        Page<PayVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<PayVo> list = page.getRecords();
        if(CollectionUtils.isNotEmpty(list)){
            //查询供应商
            List<Long> supplyIds = list.stream().map(PayVo::getSupplyId).collect(Collectors.toList());
            List<SupplierInfo> supplierInfoList = supplierInfoService.list(Wrappers.<SupplierInfo>query().lambda().in(SupplierInfo::getId,supplyIds));
            Map<Long,List<SupplierInfo>> supplyMap = supplierInfoList.stream().distinct().collect(Collectors.groupingBy(SupplierInfo::getId));
            //查询付款状态
            List<Long> ids = list.stream().map(PayVo::getId).collect(Collectors.toList());
            List<AccountPayment> accountPaymentList = accountPaymentService.list(Wrappers.<AccountPayment>query().lambda().in(AccountPayment::getBusinessId,ids));
            Map<Long,List<AccountPayment>> accountPaymentMap = accountPaymentList.stream().distinct().collect(Collectors.groupingBy(AccountPayment::getBusinessId));
            for(PayVo p:list){
                if(MapUtils.isNotEmpty(supplyMap)){
                    List<SupplierInfo> supplys = supplyMap.getOrDefault(p.getSupplyId(),null);
                    p.setSupplyName(supplys==null?null:supplys.get(0).getName());
                }
                if(MapUtils.isNotEmpty(accountPaymentMap)){
                    List<AccountPayment> accountPayments = accountPaymentMap.getOrDefault(p.getId(),null);
                    p.setPaymentStatus(accountPayments==null?null:accountPayments.get(0).getStatus());
                }
            }
        }
        return page;
    }

    @Override
    public PayVo detail(Long id) {
        Pay Pay = this.getById(id);
        PayVo result = BeanUtil.toBean(Pay, PayVo.class);
        return result;
    }

    @Override
    public void add(PayDto payDto) {
        this.save(payDto);
    }

    @Override
    public void edit(PayDto payDto) {
        this.updateById(payDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 采购付款统计
     * @param dto
     * @return
     */
    @Override
    public Map<String, Object> payStatistics(PaySelectDto dto) {
        List<Integer> statusList =  Arrays.asList(0,1,2);//付款状态  0未付款  1部分付款  2已付款'

        //存放付款统计数据
        Map<String,Object> map = new HashMap<>();

        //存放付款统计详情列表
        List<Map<String, Object>> list = new ArrayList<>();

        //查询付款详情
        QueryWrapper<Pay> query = Wrappers.query();
        query.select("ifnull(pay_status,-1) payStatus,ifnull(amount,0) amount");
        query.groupBy("pay_status");
        List<Pay> pays = baseMapper.selectList(query);
        Map<String, List<Pay>> payMap = pays.stream().collect(Collectors.groupingBy(Pay::getPayStatus));

        //计算合计金额
        BigDecimal amount = pays.stream().map(Pay::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        map.put("amount",amount);

        //赋值每个付款状态的金额
        for (Integer status : statusList) {
            //赋初始值
            Map<String,Object> statusMap = new HashMap<>();
            statusMap.put("payStatus",status);
            statusMap.put("payAmount",0);
            //赋值
            List<Pay> payList = payMap.get(status);
            if (CollectionUtils.isNotEmpty(payList)){
                statusMap.put("payAmount",payList.get(0).getAmount());
            }
            list.add(statusMap);
        }
        map.put("payList",list);
        return map;
    }

}
